#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ci_utils import (
    amqp_utils,
    amqp_worker,
)
from ci_utils.data_store import DataStoreException
from coverageretriever import (
    CoverageRetrieverException,
    retry,
    snip_coverage_xml,
)

SUCCESS_TEMPLATE = u'Deposited coverage.xml into swift for ticket %s.'
MISSING_PARAMS_TEMPLATE = (u'Failed to handle request: missing build log '
                           'filename for ticket %s')
PARSE_FAILURE_TEMPLATE = u'Failed to parse build log for ticket %s: %s'
SWIFT_GET_FAILURE_TEMPLATE = (u'Failed to retrieve build log from swift '
                              'for ticket %s: %s')
SWIFT_PUT_FAILURE_TEMPLATE = (u'Failed to put build log into swift for '
                              'ticket %s: %s')


class CoverageRetrieverWorker(amqp_worker.AMQPWorker):

    def __init__(self):
        super(CoverageRetrieverWorker, self).__init__('coverageretriever')

    def get_build_log(self, data_store, build_log_filename, logger=None):
        """Get build log; retry swift if our log's not there yet.

        :param data_store: DataStore from which to get log
        :param build_log_filename: identifies build log to get
        """
        # NOTE we wrap this at runtime just to pass the logger in
        @retry(DataStoreException, logger=logger)
        def get_build_log_actual(data_store, build_log_filename):
            return data_store.get_file(build_log_filename)
        return get_build_log_actual(data_store, build_log_filename)

    def handle_request(self, params, logger):
        """Snip coverage.xml from incoming build log and put in swift.

        :param params: dictionary of params from incoming message
        :param logger: logging.Logger for reporting
        """
        ticket_id = params['ticket_id']
        try:
            build_log_filename = params['build_log_filename']
        except KeyError:
            logger.error(MISSING_PARAMS_TEMPLATE, ticket_id)
            result_data = {
                'message': MISSING_PARAMS_TEMPLATE % ticket_id}
            return amqp_utils.progress_failed, result_data
        data_store = self._create_data_store(ticket_id)
        try:
            log_text = self.get_build_log(
                data_store,
                build_log_filename,
                logger
            )
        except DataStoreException as e:
            logger.error(SWIFT_GET_FAILURE_TEMPLATE, ticket_id, e.message)
            result_data = {
                'message': SWIFT_GET_FAILURE_TEMPLATE % (ticket_id, e.message)}
            return amqp_utils.progress_failed, result_data
        try:
            coverage_xml_text = snip_coverage_xml(log_text)
        except CoverageRetrieverException as e:
            logger.error(PARSE_FAILURE_TEMPLATE, ticket_id, e.message)
            result_data = {
                'message': PARSE_FAILURE_TEMPLATE % (ticket_id, e.message)}
            return amqp_utils.progress_failed, result_data
        try:
            result_link = data_store.put_file(
                'coverage.xml',
                coverage_xml_text,
                'text/xml',
            )
        except DataStoreException as e:
            logger.error(SWIFT_PUT_FAILURE_TEMPLATE, ticket_id, e.message)
            result_data = {
                'message': SWIFT_PUT_FAILURE_TEMPLATE % (ticket_id, e.message)}
            return amqp_utils.progress_failed, result_data
        logger.info(SUCCESS_TEMPLATE, ticket_id)
        result_data = {
            'message': SUCCESS_TEMPLATE % ticket_id,
            'result_link': result_link,
        }
        return amqp_utils.progress_completed, result_data


if __name__ == '__main__':
    CoverageRetrieverWorker().main(amqp_utils.COVERAGE_RETRIEVER_QUEUE)
