# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from mock import patch, Mock

from ci_utils import amqp_utils
from ci_utils.data_store import DataStoreException
from ci_utils.testing.fixtures import FakeDataStore
from coverageretriever import CoverageRetrieverException
from coverageretriever.tests import (
    CoverageRetrieverWorkerTestCase,
    vanilla_build_log,
)
from coverageretriever.run_worker import (
    CoverageRetrieverWorker,
    MISSING_PARAMS_TEMPLATE,
    PARSE_FAILURE_TEMPLATE,
    SUCCESS_TEMPLATE,
    SWIFT_GET_FAILURE_TEMPLATE,
    SWIFT_PUT_FAILURE_TEMPLATE,
)


class HandleRequestTestCase(CoverageRetrieverWorkerTestCase):

    def test_worker_handles_request_successfully(self):
        """Large integration test for coverage retrieval."""
        self.fake_logger = Mock(spec_set=logging.Logger)
        with self.patch_create_data_store():
            self.put_fake_build_log()
            worker = CoverageRetrieverWorker()
            fake_params = {'build_log_filename': 'build.log',
                           'ticket_id': 'fake_ticket_id'}
            amqp_result, result_data = worker.handle_request(fake_params,
                                                             self.fake_logger)
            self.assertEqual(amqp_utils.progress_completed, amqp_result)
            self.assertEqual(
                SUCCESS_TEMPLATE % 'fake_ticket_id',
                result_data['message'])
            self.fake_logger.info.assert_called_with(
                SUCCESS_TEMPLATE, 'fake_ticket_id')
            coverage_xml = self.fake_data_store.get_file('coverage.xml')
            self.validate_coverage_xml(coverage_xml)

    def test_missing_params_raise(self):
        """Missing params raise an exception."""
        self.fake_logger = Mock(spec_set=logging.Logger)
        worker = CoverageRetrieverWorker()
        malevolent_params = {'gibberish_filename': 'build.log',
                             'ticket_id': 'fake_ticket_id'}
        amqp_result, result_data = worker.handle_request(malevolent_params,
                                                         self.fake_logger)
        self.assertEqual(amqp_utils.progress_failed, amqp_result)
        self.assertEqual(
            MISSING_PARAMS_TEMPLATE % 'fake_ticket_id',
            result_data['message'])
        self.fake_logger.error.assert_called_with(
            MISSING_PARAMS_TEMPLATE, 'fake_ticket_id')

    @patch('coverageretriever.run_worker.snip_coverage_xml')
    def test_coverage_xml_not_found(self, snip_coverage_xml):
        """Return failure if coverage.xml not found in build log."""
        self.fake_logger = Mock(spec_set=logging.Logger)
        with self.patch_create_data_store():
            self.put_fake_build_log()
            snip_coverage_xml.side_effect = CoverageRetrieverException()
            worker = CoverageRetrieverWorker()
            fake_params = {'build_log_filename': 'build.log',
                           'ticket_id': 'fake_ticket_id'}
            amqp_result, result_data = worker.handle_request(fake_params,
                                                             self.fake_logger)
            self.assertEqual(amqp_utils.progress_failed, amqp_result)
            # NOTE that the underlying exception doesn't say much,
            # update when that's remedied
            self.assertEqual(
                PARSE_FAILURE_TEMPLATE % ('fake_ticket_id', ''),
                result_data['message'])
            self.fake_logger.error.assert_called_with(
                PARSE_FAILURE_TEMPLATE, 'fake_ticket_id', '')

    @patch.object(CoverageRetrieverWorker, 'get_build_log')
    def test_build_log_not_in_swift(self, get_build_log):
        """Return failure if swift fails to yield our build log."""
        self.fake_logger = Mock(spec_set=logging.Logger)
        with self.patch_create_data_store():
            self.put_fake_build_log()
            get_build_log.side_effect = DataStoreException()
            worker = CoverageRetrieverWorker()
            fake_params = {'build_log_filename': 'build.log',
                           'ticket_id': 'fake_ticket_id'}
            amqp_result, result_data = worker.handle_request(fake_params,
                                                             self.fake_logger)
            self.assertEqual(amqp_utils.progress_failed, amqp_result)
            # NOTE that the underlying exception doesn't say much,
            # update when that's remedied
            self.assertEqual(
                SWIFT_GET_FAILURE_TEMPLATE % ('fake_ticket_id', ''),
                result_data['message'])
            self.fake_logger.error.assert_called_with(
                SWIFT_GET_FAILURE_TEMPLATE, 'fake_ticket_id', '')

    @patch.object(FakeDataStore, 'get_file')
    def test_retry_get_build_log_from_swift(self, get_file):
        """Retry getting build log from swift."""
        self.fake_logger = Mock(spec_set=logging.Logger)
        with self.patch_create_data_store():
            get_file.side_effect = [DataStoreException("No log!"),
                                    DataStoreException("Still no log!"),
                                    vanilla_build_log]
            worker = CoverageRetrieverWorker()
            fake_params = {'build_log_filename': 'build.log',
                           'ticket_id': 'fake_ticket_id'}
            amqp_result, result_data = worker.handle_request(fake_params,
                                                             self.fake_logger)
            self.assertEqual(amqp_utils.progress_completed, amqp_result)
            self.assertEqual(
                SUCCESS_TEMPLATE % 'fake_ticket_id',
                result_data['message'])
            self.fake_logger.info.assert_called_with(
                SUCCESS_TEMPLATE, 'fake_ticket_id')
            self.fake_logger.warning.assert_called_with(
                "Still no log!, retrying in 6 seconds. . . .")
            self.assertEqual(3, get_file.call_count)

    def test_put_coverage_xml_into_swift_failure_raises(self):
        """Return failure if we fail to put coverage.xml into swift."""
        self.fake_logger = Mock(spec_set=logging.Logger)
        with self.patch_create_data_store():
            self.put_fake_build_log()
            # wait 'til we've deposited our dummy before patching
            with patch.object(FakeDataStore, 'put_file') as put_file:
                put_file.side_effect = DataStoreException()
                worker = CoverageRetrieverWorker()
                fake_params = {'build_log_filename': 'build.log',
                               'ticket_id': 'fake_ticket_id'}
                amqp_result, result_data = worker.handle_request(
                    fake_params,
                    self.fake_logger
                )
                self.assertEqual(amqp_utils.progress_failed, amqp_result)
                # NOTE that the underlying exception doesn't say much,
                # update when that's remedied
                self.assertEqual(
                    SWIFT_PUT_FAILURE_TEMPLATE % ('fake_ticket_id', ''),
                    result_data['message'])
                self.fake_logger.error.assert_called_with(
                    SWIFT_PUT_FAILURE_TEMPLATE, 'fake_ticket_id', '')
